/*
 * dstumbler v0.3 [dstumbler.h]
 * by h1kari - (c) Dachb0den Labs 2001
 */

/*
 * Copyright (c) 2001 Dachb0den Labs.
 *      David Hulton <h1kari@dachb0den.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by David Hulton.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY David Hulton AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL David Hulton OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/types.h>
#ifdef __OpenBSD__
#include <sys/cdefs.h>
#include <sys/param.h>
#endif
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <net/if.h>

#include <sys/termios.h>

#ifdef __FreeBSD__
#include <net/if_var.h>
#include <net/ethernet.h>
#include <machine/if_wavelan_ieee.h>
#else
#include <netinet/in.h>
#include <netinet/if_ether.h>
#ifdef __NetBSD__
#include <dev/ic/wi_ieee.h>
#else
#ifdef __OpenBSD__
#ifdef OPENBSD_IC_WAVELAN
#include <dev/ic/if_wi_ieee.h>
#else
#include <dev/pcmcia/if_wavelan_ieee.h>
#endif
#else
#include <dev/pcmcia/if_wavelan_ieee.h>
#endif
#endif
#endif

#ifdef __OpenBSD__
#include <curses.h>
#else
#include <ncurses.h>
#endif

#include "config.h"

/*
 * definitions
 */
#define MIN_SCREEN_Y    24
#define MIN_SCREEN_X    78
#define SSIDLEN         16
#define MACSIZE         6
#define QUALITYSIZE     3
#define PREALLOC_APS    512
#define PREALLOC_LOG    128
#define GRAPH_NORM_MAX  150
#define GRAPH_PRISM_MAX 200
#define BANNER          "dstumbler v0.3 by h1kari - " \
 "(c) Dachb0den Labs 2001"
#define FILE_FMT        "%15s %17s %02d %02d:%02d:%02d %1d " \
 "%010ld.%010ld %010ld.%010ld\n"
#define FILE_PROMPT     "File: "
#define FILE_BUFFERLEN  (ALERT_SCR_COLS - strlen(FILE_PROMPT) - 4)
#define MENU \
 " [ basic navigation ]---\n" \
 " [+/-]: move up/down\n" \
 " [u/d]: page up/down\n" \
 " [e/h]: end/home\n" \
 " [n]: newest\n" \
 " [s]: sort\n" \
 " [a]: autoselect new aps\n" \
 " [r]: resolve mfg\n\n" \
 " [ file commands ]------\n" \
 " [l]: load from file\n" \
 " [b]: backup to file\n" \
 " [q]: quit\n"

/*
 * macros
 */
#define APCHANGE(x)\
{\
  aps_cur = x;\
  apchange++;\
  apnew = 0;\
}

#define APNEW(x)\
{\
  aps_new = x;\
\
  if(autosel)\
  {\
    if(x != aps_lst)\
    {\
      apnew++;\
      apchange = 0;\
    }\
    aps_lst = x;\
  }\
}

#ifndef MAX
#define MAX(x, y) (x > y ? x : y)
#endif
#ifndef MIN
#define MIN(x, y) (x < y ? x : y)
#endif
#define SWAP(x, y) { x ^= y; y ^= x; x ^= y; }

/*
 * define all of these as macros for now, for easy implementation of other
 * card driver symantics in the future.
 */
#define getssid(x, y) wi_getstat(x, WI_RID_CURRENT_SSID, y, SSIDLEN)
#define getbssid(x, y) wi_getstat(x, WI_RID_CURRENT_BSSID, (char *)y, MACSIZE)
#define getchan(x, y) wi_getstat(x, WI_RID_CURRENT_CHAN, (char *)y, 0)
#define getquality(x, y) wi_getstat(x, WI_RID_COMMS_QUALITY, (char *)y, 0)
#define getcrypt(x, y) wi_getstat(x, WI_RID_ENCRYPTION, (char *)y, 0)
#define getcryptkeys(x, y) wi_getstat(x, WI_RID_DEFLT_CRYPT_KEYS, (char *)y, 0)
#define getdefaultssid(x, y) wi_getstat(x, WI_RID_OWN_SSID, y, SSIDLEN)
#define getmacaddr(x, y) wi_getstat(x, WI_RID_MAC_NODE, (char *)y, MACSIZE)
#define setcrypt(x, y) wi_setstat(x, WI_RID_ENCRYPTION, (char *)y, 2)
#define setcryptkeys(x, y) wi_setstat(x, WI_RID_DEFLT_CRYPT_KEYS, (char *)y, \
 (sizeof(struct wi_ltv_keys) / 2) + 1);
#define setdefaultssid(x, y) wi_setstat(x, WI_RID_OWN_SSID, y, SSIDLEN);
#define setmacaddr(x, y) wi_setstat(x, WI_RID_MAC_NODE, (char *)y, MACSIZE);
#define getgps(x, y) {\
 memcpy((char *)x, (char *)&ns, sizeof(struct gps_s));\
 memcpy((char *)y, (char *)&ew, sizeof(struct gps_s));\
}

/* redefine these for compatibility with freebsd/openbsd */
#ifdef __FreeBSD__
#define htole16(x) (x)
#define le16toh(x) (x)
#define O_SYNC     O_FSYNC
#else
#ifdef __OpenBSD__
#define le16toh(x) letoh16(x)
#endif
#endif

struct gps_s {
  char dir;
  float coordinates;
};

struct log_s {
  struct gps_s ns;
  struct gps_s ew;
  int quality[QUALITYSIZE];
  struct timeval seen;
};

struct aps_s {
  struct gps_s ns;
  struct gps_s ew;
  char ssid[SSIDLEN];
  u_char bssid[MACSIZE];
  int chan;
  int quality[QUALITYSIZE];
  int weped;
  struct timeval firstseen;
  struct timeval lastseen;
  struct log_s **log;
  int log_len;
  int log_max;
};

struct mfg_s {
  char mac[MFG_MACSIZE];
  char *string;
};

/*
 * declarations
 */
WINDOW *main_scr, *ap_scr, *info_scr, *graph_scr, *menu_scr, *alert_scr;
WINDOW *top_border, *top_right_border, *right_border, *bottom_border;
int max_lines, max_cols;
int prism2, autosel, resolvmfg, macreset, ncurse, nodevice, usegps;
int apchange, apnew;
int aps_len, aps_cur, aps_new, aps_win, aps_max, aps_lst;
int gps_fd;
char *device, *gps;
struct aps_s **aps;
struct gps_s ns, ew;

struct {
  int ap_scr;
  int info_scr;
  int graph_scr;
  int menu_scr;
  int alert_scr;
} redraw;

struct {
  int wi_started;
  int crypt;
  u_char macaddr[MACSIZE];
  struct wi_ltv_keys keys;
  char defaultssid[SSIDLEN];

  int gps_started;
  struct termios ttyset;
} backup;

/*
 * main.c
 */
int main(int, char **);
void print_usage(char *);
void init_curses(void);
void draw_screen(void);
void init_vars(void);
void exitclean(int);

/*
 * misc.c
 */
void mfgresolve(char *, char *);
void smart_redraw(void);
void printbanner(void);
void alert(const char *, ...);

/*
 * aps.c
 */
void poll_aps(const char *);
int find_ap(const char *, const char *, int);
void redraw_aps(void);
void redraw_apinfo(void);
void redraw_graph(void);
void print_graph(int *, int);
void add_ap(const struct aps_s *);
void add_log(struct aps_s *, const struct aps_s *);

/*
 * menu.c
 */
void parse_cmd(int);
void sort_aplist(void);
int aplistcmp(const void *, const void *);
void redraw_menu(void);
void load_aplist(void);
void backup_aplist(void);
void fileprompt(char *, int);

/*
 * wistat.c
 */
void wi_ioctl(const char *, struct wi_req *, int);
void wi_getstat(const char *, int, char *, int len);
void wi_setstat(const char *, int, char *, int len);
void wi_strncpyval(char *, const char *, int);
void wi_start(const char *);
void wi_stop(const char *);

/*
 * gps.c
 */
int gps_open(const char *device);
void gps_close(int fd);
void gps_parse(int fd);
void gps_start(const char *device);
void gps_stop(void);
