/*
 * dstumbler v0.3 [main.c]
 * by h1kari - (c) Dachb0den Labs 2001
 */

/*
 * Copyright (c) 2001 Dachb0den Labs.
 *      David Hulton <h1kari@dachb0den.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by David Hulton.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY David Hulton AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL David Hulton OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>

#ifdef __OpenBSD__
#include <curses.h>
#else
#include <ncurses.h>
#endif

#include "dstumbler.h"
#include "screen.h"

extern char *optarg;

/*
 * parse arguments and jump to ap scan loop
 */
int
main(int argc, char **argv)
{
  int c;

  if(argc < 2)
    print_usage(argv[0]);

  init_vars();

  if(argv[1][0] == '-' && argv[1][1] != 'd')
    print_usage(argv[0]);
  else if(strcmp(argv[1], "-d") != 0)
  {
    optind++;
    device = argv[1];
  }

  while((c = getopt(argc, argv, "pnmdg:")) != -1)
  {
    switch(c)
    {
      case 'p':
        prism2 = 1;
        break;
      case 'n':
        ncurse = 0;
        break;
      case 'm':
        macreset = 1;
        break;
      case 'd':
        nodevice = 1;
        break;
      case 'g':
        usegps = 1;
        gps = optarg;
        break;
      case '?':
      case 'h':
      default:
        print_usage(argv[0]);
    }
  }

  if(nodevice)
    device = NULL;

  init_curses();

  redraw_menu();
  printbanner();
  smart_redraw();

  if(!nodevice)
    wi_start(device);

  if(usegps)
    gps_start(gps);

  poll_aps(nodevice ? NULL : device);

  exitclean(0); 

  return -1;
}

/*
 * print all of the usage info and exit
 */
void
print_usage(char *progname)
{
  fprintf(stderr,
   "usage: %s <device> [-pnmd] [-g <gps device>]\n"
   " -p: specify the use of a prism2 card\n"
   " -n: use basic ascii characters for limited terminal fonts\n"
   " -m: randomly set mac address before scanning\n"
   " -d: run dstumbler without specifying a wireless device\n"
   " -g: specify gps device to use\n", progname);
  exit(2);
}

/*
 * initialize curses and draw windows
 */
void
init_curses(void)
{
  main_scr = initscr();

  signal(SIGHUP, exitclean);
  signal(SIGINT, exitclean);
  signal(SIGKILL, exitclean);
  signal(SIGPIPE, exitclean);
  signal(SIGALRM, exitclean);
  signal(SIGTERM, exitclean);

  keypad(stdscr, TRUE);
  nodelay(stdscr, TRUE);
  cbreak();
  nonl();
  noecho();
  curs_set(0);

  getmaxyx(main_scr, max_lines, max_cols);

  /*
   * initialize colors if user is using a color terminal
   */
  if(has_colors())
  {
    use_default_colors();
    start_color();
    init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_DEFAULT);
    init_pair(COLOR_RED, COLOR_RED, COLOR_DEFAULT);
    init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_DEFAULT);
    init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_DEFAULT);
    init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_DEFAULT);
    init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_DEFAULT);
    init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_DEFAULT);
  }

  if(max_lines < MIN_SCREEN_Y || max_cols < MIN_SCREEN_X)
  {
    wprintw(main_scr, "error: sorry, your screen size must be at least %dx%d\n"
     "press any key to exit..\n", MIN_SCREEN_Y, MIN_SCREEN_X);
    getch();
    exitclean(2);
  }

  draw_screen();
}

/*
 * draw/redraw all windows
 */
void
draw_screen(void)
{
  int i;

  getmaxyx(main_scr, max_lines, max_cols);

  /*
   * initialize and draw borders
   */
  top_border = subwin(main_scr, TOP_BORDER_LINES, TOP_BORDER_COLS,
   TOP_BORDER_Y, TOP_BORDER_X);
  wattrset(top_border, BORDER_COLOR);
  for(i = 0; i < AP_SCR_COLS; i++)
    waddch(top_border, ncurse ? ACS_HLINE : BORDER_HLINE);
  waddch(top_border, ncurse ? ACS_PLUS : BORDER_PLUS);
  for(i = 0; i < INFO_SCR_COLS; i++)
    waddch(top_border, ncurse ? ACS_HLINE : BORDER_HLINE);

  top_right_border = subwin(main_scr, TOP_RIGHT_BORDER_LINES,
   TOP_RIGHT_BORDER_COLS, TOP_RIGHT_BORDER_Y, TOP_RIGHT_BORDER_X);
  wattrset(top_right_border, BORDER_COLOR);
  for(i = 0; i < TOP_RIGHT_BORDER_LINES; i++)
    waddch(top_right_border, ncurse ? ACS_VLINE : BORDER_VLINE);

  right_border = subwin(main_scr, RIGHT_BORDER_LINES, RIGHT_BORDER_COLS,
   RIGHT_BORDER_Y, RIGHT_BORDER_X);
  wattrset(right_border, BORDER_COLOR);
  for(i = 0; i < RIGHT_BORDER_LINES; i++)
    waddch(right_border, ncurse ? ACS_VLINE : BORDER_VLINE);

  bottom_border = subwin(main_scr, BOTTOM_BORDER_LINES, BOTTOM_BORDER_COLS,
   BOTTOM_BORDER_Y, BOTTOM_BORDER_X);
  wattrset(bottom_border, BORDER_COLOR);
  for(i = 0; i < GRAPH_SCR_COLS; i++)
    waddch(bottom_border, ncurse ? ACS_HLINE : BORDER_HLINE);
  waddch(bottom_border, ncurse ? ACS_BTEE : BORDER_BTEE);
  for(i = 0; i < MENU_SCR_COLS; i++)
    waddch(bottom_border, ncurse ? ACS_HLINE : BORDER_HLINE);

  /*
   * now initialize and set options for the other subwindows
   */
  ap_scr = subwin(main_scr, AP_SCR_LINES, AP_SCR_COLS, AP_SCR_Y, AP_SCR_X);
  info_scr = subwin(main_scr, INFO_SCR_LINES, INFO_SCR_COLS, INFO_SCR_Y,
   INFO_SCR_X);
  graph_scr = subwin(main_scr, GRAPH_SCR_LINES, GRAPH_SCR_COLS, GRAPH_SCR_Y,
   GRAPH_SCR_X);
  wattrset(graph_scr, GRAPH_COLOR);
  menu_scr = subwin(main_scr, MENU_SCR_LINES, MENU_SCR_COLS, MENU_SCR_Y,
   MENU_SCR_X);
  alert_scr = subwin(main_scr, ALERT_SCR_LINES, ALERT_SCR_COLS, ALERT_SCR_Y,
   ALERT_SCR_X);

  scrollok(ap_scr, TRUE);
  scrollok(graph_scr, TRUE);

  refresh();
}

/*
 * make sure all the global variables are initialized properly
 */
#define APS_INIT_SIZE (sizeof(struct aps_s *) * PREALLOC_APS)
void
init_vars(void)
{
  /*
   * pre-allocate PREALLOC_APS amount of pointers for the aps array and zero
   * it out so there isn't any confusion.
   */
  if((aps = (struct aps_s **)malloc(APS_INIT_SIZE)) == NULL)
  {
    fprintf(stderr, "error: unable to allocate memory: %s\n", strerror(errno));
    exit(2);
  }

  memset((char *)aps, 0, APS_INIT_SIZE);

  /*
   * aps_max contains the current amount of bytes allocated for aps
   * aps_len is the current amount of allocated aps in the array
   * and aps_cur and aps_new contain the index of their corresponding
   * aps.
   */
  aps_max = PREALLOC_APS;
  aps_len = aps_cur = aps_new = aps_lst = aps_win = 0;

  /* flags for when aps are added */
  apchange = apnew = 0;

  /*
   * set redraw on all windows to 0, if they're set to 1 in other functions
   * when smart_redraw is called, the corresponding windows will be updated
   */
  redraw.ap_scr = redraw.info_scr = redraw.graph_scr = redraw.menu_scr =
   redraw.alert_scr = 0;

  /* backup previous nic info */
  backup.wi_started = 0;
  backup.gps_started = 0;

  /* command line options :/ */
  prism2 = 0;
  macreset = 0;
  ncurse = 1; /* default to full ascii character set */
  autosel = 0;
  resolvmfg = 0;
  nodevice = 0;
  usegps = 0;

  /* set default gps stuff */
  gps = NULL;

  ns.dir = 'N';
  ns.coordinates = 0;

  ew.dir = 'E';
  ew.coordinates = 0;
}

/*
 * make sure if we receive a signal or ctrl+c or something, we exit cleanly
 * otherwise ncurses will go apeshit.
 */
void
exitclean(int val)
{
  int i, j;

  endwin();

  if(backup.wi_started)
    wi_stop(device);

  if(backup.gps_started)
    gps_stop();

  /* free all of our malloc'ed memory */
  for(i = 0; i < aps_len; i++)
  {
    for(j = 0; j < aps[i]->log_len; j++)
      free(aps[i]->log[j]);

    free(aps[i]->log);
    free(aps[i]);
  }

  free(aps);

  putchar('\n');
  exit(val);
}
