/*
 * dstumbler v0.3 [menu.c]
 * by h1kari - (c) Dachb0den Labs 2001
 */

/*
 * Copyright (c) 2001 Dachb0den Labs.
 *      David Hulton <h1kari@dachb0den.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by David Hulton.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY David Hulton AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL David Hulton OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifdef __OpenBSD__
#include <curses.h>
#else
#include <ncurses.h>
#endif

#include "dstumbler.h"
#include "screen.h"

#define REDRAWAP() {\
 if(aps_len > 0)\
 {\
   redraw_aps();\
   redraw_apinfo();\
   redraw_graph();\
 }\
}

/*
 * parse commands by the user
 */
void
parse_cmd(int c)
{
  switch(c)
  {
    /* for up key */
    case KEY_UP:
    case '[':
    case '8':
    case '+':
    case '=':
      APCHANGE(aps_cur > 0 ? aps_cur - 1 : aps_cur);
      REDRAWAP();
      break;
    case KEY_DOWN:
    case ']':
    case '2':
    case '-':
      APCHANGE(aps_cur < (aps_len - 1) ? aps_cur + 1 : aps_cur);
      REDRAWAP();
      break;
    case KEY_PPAGE:
    case 'u':
      APCHANGE(aps_cur > AP_SCR_LINES ? aps_cur - AP_SCR_LINES : 0);
      REDRAWAP();
      break;
    case KEY_NPAGE:
    case 'd':
      APCHANGE(aps_cur < (aps_len - (AP_SCR_LINES + 1)) ?
       aps_cur + AP_SCR_LINES : aps_len - 1);
      REDRAWAP();
      break;
    case KEY_END:
    case KEY_RIGHT:
    case 'e':
      APCHANGE(aps_len > 0 ? aps_len - 1 : 0);
      REDRAWAP();
      break;
    case KEY_HOME:
    case KEY_LEFT:
    case 'h':
      APCHANGE(0);
      REDRAWAP();
      break;
    case 'n':
    case 'N':
      APCHANGE(aps_new);
      REDRAWAP();
      break;
    case 's':
    case 'S':
      sort_aplist();
      break;
    case 'a':
    case 'A':
      autosel++;
      autosel %= 2;
      break;
    case 'r':
    case 'R':
      resolvmfg++;
      resolvmfg %= 2;

      if(aps_len > 0)
        redraw_aps();

      break;
    case 'l':
    case 'L':
      load_aplist();
      break;
    case 'b':
    case 'B':
      backup_aplist();
      break;
    case 'q':
    case 'Q':
      exitclean(0);
      break;
    default:
      break;
  }
}

/*
 * sort the aplist by the most recently seen aps
 */
void
sort_aplist(void)
{
  if(aps_len > 1)
  {
    /* sort out all the aps based on time */
    if(heapsort(aps, aps_len, sizeof(struct aps_s *), &aplistcmp) == -1)
    {
      alert("error: problem sorting aps: %s", strerror(errno));
      return;
    }

    APNEW(0);
    APCHANGE(0);

    REDRAWAP();
  }
}

/*
 * compare two ap pointers, this is kinda switched so we can sort decending
 * instead of ascending
 */
int
aplistcmp(const void *a1, const void *b1)
{
  struct aps_s *a, *b;

  a = (struct aps_s *)(*(void **)a1);
  b = (struct aps_s *)(*(void **)b1);

  /*
   * if a.sec == b.sec and a.usec == b.usec, then 0
   * if a.sec == b.sec and a.usec > b.usec, then 1
   * if a.sec == b.sec and a.usec < b.usec, then -1
   * if a.sec > b.sec, then 1
   * if a.sec < b.sec, then -1
   */
  return(a->lastseen.tv_sec == b->lastseen.tv_sec ?
   (a->lastseen.tv_usec == b->lastseen.tv_usec ? 0 :
   (a->lastseen.tv_usec > b->lastseen.tv_usec ? -1 : 1)) :
   (a->lastseen.tv_sec > b->lastseen.tv_sec ? -1 : 1));
}

/*
 * draw menu of key options
 */
void
redraw_menu(void)
{
  int i, mode = 0;
  char menu[] = MENU;

  for(i = 0; i < strlen(menu); i++)
  {
    if(menu[i] == '[')
      mode = 1;
    else if(menu[i] == ']')
      mode = 0;

    if(mode && menu[i] != '[')
      waddch(menu_scr, menu[i] | MENU_COLOR_BOLD);
    else
      waddch(menu_scr, menu[i] | MENU_COLOR);
  }

  redraw.menu_scr++;
}

/*
 * load ap list from a file
 */
#define DATABUF_LEN 256
void
load_aplist(void)
{
  int i, found;
  u_int temp_mac[MACSIZE];
  char buffer[FILE_BUFFERLEN], databuf[DATABUF_LEN];
  struct aps_s new;
  FILE *fd;

  /*
   * first get the name of the file to import from
   */
  fileprompt(buffer, FILE_BUFFERLEN);

  /*
   * now loop through the file and import to the end of the aplist
   */
  if((fd = fopen(buffer, "r")) == NULL)
  {
    alert("error: unable to open specified file: %s", strerror(errno));
    return;
  }

  memset((char *)&new, 0, sizeof(struct aps_s));

  while(!feof(fd))
  {
    fgets(databuf, DATABUF_LEN - 1, fd);
    databuf[strlen(databuf) - 1] = '\0';

    if(databuf[0] == '#')
      continue;

    if(sscanf(databuf, "%c %f %c %f ( %[^)]) %*s "
     "( %2x:%2x:%2x:%2x:%2x:%2x ) %*d:%*d:%*d (GMT) [ %d %d %d ] "
     "# %d %d %ld.%ld %*d",
     &new.ns.dir, &new.ns.coordinates, &new.ew.dir, &new.ew.coordinates,
     new.ssid, &temp_mac[0], &temp_mac[1], &temp_mac[2], &temp_mac[3],
     &temp_mac[4], &temp_mac[5], &new.quality[0], &new.quality[1],
     &new.quality[2], &new.weped, &new.chan, &new.lastseen.tv_sec,
     &new.lastseen.tv_usec) < 9)
    {
      alert("error: line in backup file is corrupt or not compatible");
      fclose(fd);
      exitclean(2);
    }

    for(i = 0; i < MACSIZE; i++)
      new.bssid[i] = temp_mac[i];

    if(new.ssid[strlen(new.ssid) - 1] == ' ')
      new.ssid[strlen(new.ssid) - 1] = '\0';

    if((found = find_ap(new.ssid, new.bssid, new.chan)) > -1)
      add_log(aps[found], &new);
    else if(found == -1)
    {
      new.firstseen.tv_sec = new.lastseen.tv_sec;
      new.firstseen.tv_usec = new.lastseen.tv_usec;
      add_ap(&new);

      memset((char *)&new, 0, sizeof(struct aps_s));
    }
  }

  fclose(fd);

  apchange++;
  REDRAWAP();
}

/*
 * save ap list to a file
 */
#define DATEBUF_LEN 33
void
backup_aplist(void)
{
  int i, j;
  char buffer[FILE_BUFFERLEN], datebuf[DATEBUF_LEN];
  time_t curr_time = time(NULL);
  struct log_s *logptr;
  struct aps_s *ptr;
  FILE *fd;

  /*
   * first we need the name of the file to export to
   */
  fileprompt(buffer, FILE_BUFFERLEN);

  /*
   * now loop through ap list and write to the file
   */
  if((fd = fopen(buffer, "w")) == NULL)
  {
    alert("unable to open specified file: %s", strerror(errno));
    return;
  }

  strftime(datebuf, DATEBUF_LEN - 1, "%Y-%m-%d", gmtime((time_t *)&curr_time));

  /* print file header */
  fprintf(fd,
   "# $Creator: dstumbler v0.3\n"
   "# $Format: wi-scan with extensions\n"
   "# Latitude\tLongitude\t( SSID )\tType\t( BSSID )\tTime (GMT)\t"
   "[ SNR Sig Noise ]\t# WEPed\tChan\tEpochTime\tPollspeed\n"
   "# $DateGMT: %s\n", datebuf);

  for(i = 0; i < aps_len; i++)
  {
    ptr = aps[i];

    for(j = 0; j < aps[i]->log_len; j++)
    {
      logptr = aps[i]->log[j];

      strftime(datebuf, 32, "%H:%M:%S (GMT)",
       gmtime((time_t *)&logptr->seen.tv_sec));

      fprintf(fd, "%c %4.7f\t%c %4.7f\t( %s )\tBBS\t"
       "( %02x:%02x:%02x:%02x:%02x:%02x )\t"
       "%s\t[ %d %d %d ]\t# %d\t%d\t%ld.%ld\t%d\n",
       logptr->ns.dir, logptr->ns.coordinates,
       logptr->ew.dir, logptr->ew.coordinates,
       ptr->ssid, ptr->bssid[0], ptr->bssid[1], ptr->bssid[2],
       ptr->bssid[3], ptr->bssid[4], ptr->bssid[5], datebuf, 
       logptr->quality[0], logptr->quality[1], logptr->quality[2],
       ptr->weped, ptr->chan, logptr->seen.tv_sec, logptr->seen.tv_usec,
       POLLSPEED);
    }
  }
  fclose(fd); 
}

/*
 * prompt for filename in the alert screen
 */
void
fileprompt(char *buffer, int len)
{
  /*
   * first we need the name of the file to export to
   */
  werase(alert_scr);
  wattrset(alert_scr, PROMPT_COLOR);
  wprintw(alert_scr, FILE_PROMPT"[ ");
  mvwaddch(alert_scr, 0, ALERT_SCR_COLS - 1, ']');
  redraw.alert_scr++;
  smart_redraw();   
    
  wattrset(alert_scr, PROMPT_COLOR_BOLD);
  echo();
  mvwgetnstr(alert_scr, 0, 8, buffer, len - 5);
  noecho();

  printbanner();
}
